@@ -149,7 +149,7 @@ def print_one_row_from_tf_dataset(test_ds):
                 for labels, value in batch.items():
                     print("{:40s}: {}".format(labels, value.numpy()[:4]))
     except:
-        print(list(test_ds.as_numpy_iterator())[0])
+        print('    Error printing. Continuing...')
 #########################################################################################
 def print_one_row_from_tf_label(test_label):
     """
@@ -947,7 +947,7 @@ def get_compiled_model(inputs, meta_outputs, output_activation, num_predicts, mo
     )
     return model
 ###############################################################################
-def add_inputs_outputs_to_auto_model_body(model_body, inputs, meta_outputs, nlp_flag=False):
+def add_outputs_to_auto_model_body(model_body, meta_outputs, nlp_flag=False):
     """
     This is specially for "auto" model types only. It requires special handling.
     """
@@ -964,7 +964,7 @@ def add_inputs_outputs_to_auto_model_body(model_body, inputs, meta_outputs, nlp_
     model_body = layers.concatenate([wide, final_outputs], name='auto_concatenate_layer')
     return model_body
 #################################################################################
-def add_inputs_outputs_to_model_body(model_body, inputs, meta_outputs):
+def add_outputs_to_model_body(model_body, meta_outputs):
     ##### This is the simplest way to convert a sequential model to functional!
     for num, each_layer in enumerate(model_body.layers):
         if num == 0:
